/* integrity.c 1.1 Pawel Krawczyk <kravietz@aba.krakow.pl>
 *
 * Licensed under GPL 2.
 *
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <syslog.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <openssl/evp.h>

int md_len;

struct sums {
	unsigned char *file;
	unsigned char *mac;
	struct sums *next;
};

static int read_sums(FILE *f, struct sums **sums) {
	unsigned char *p;
	unsigned char line[300];
	struct sums *s;
	int lnum = 1;

	s = malloc(sizeof(struct sums));
	if(s == NULL) {
		perror("malloc");
		exit(1);
	}

	*sums = s;

	while(1) {
		s->next = NULL;
		s->file = NULL;
		s->mac = NULL;

		if ( fgets(line, sizeof(line)-10, f) == NULL)
			break;
		/* fgets leaves trailing newline */
		p = strrchr(line, '\n');
		if( p != NULL)
			*p = '\0';
		/* terminate line to contain only filename */
		p = strchr(line, ' ');
		if ( p != NULL ) {
			*p = '\0';
			p++;
		}

		/* fill another record of the sums structure */
		s->file = strdup(line);
		if( p == NULL )
			s->mac = NULL;
		else
			s->mac = strdup(p);
		s->next = malloc(sizeof(struct sums));
		if(s->next == NULL) {
			perror("malloc2");
			exit(1);
		}

		s = s->next;

		lnum++;

	}

	s->next = NULL;
	
	return 0;
}

unsigned char *file_md(unsigned char *file) {
	int fd = -1;
	EVP_MD_CTX ctx;
	static unsigned char buf[EVP_MAX_MD_SIZE*3];
	unsigned char md[EVP_MAX_MD_SIZE];
	int i;

	/* HASH */
	EVP_DigestInit(&ctx, EVP_md5());
	/* HASH */
	/* Instead of EVP_md5 you could use EVP_sha1 */

	fd = open(file, O_RDONLY);
	if(fd < 0) {
		fprintf(stderr, "%s: %s\n", file, strerror(errno));
		return NULL;
	}

	while(1) {
		int r;

		r = read(fd, buf, sizeof(buf));
		if(r <= 0)
			break;
		EVP_DigestUpdate(&ctx, buf, r);
	}

	close(fd);

	EVP_DigestFinal(&ctx, md, &md_len);

	for(i=0; i<md_len; i++) {
		snprintf(buf+i, sizeof(buf), "%02x", md[i]);
	}
	buf[md_len*2] = '\0';


	return buf;
}

int main(int argc, char **argv) {
	struct sums *sums = NULL;
	int calculate_md = 0;
	char command[1000];

	memset(command, 0, sizeof(command));

	while( argc > 1) {
		argc--;
		argv++;

		if(strncmp(*argv, "-c", 2) == 0)
			calculate_md = 1;
		else
			strncpy(command, *argv, sizeof(command)-1);

	}

	read_sums(stdin, &sums);

	setpriority(PRIO_PROCESS, 0, 15);

	OpenSSL_add_all_digests();

	while(1) {

		struct sums *s = sums;

		while(s != NULL && s->file != NULL) {
			unsigned char *new_md;

			new_md = file_md(s->file);
			if(s->mac == NULL) {
				if(calculate_md) {
					printf("%s %s\n", s->file, new_md);
				}
				else {
					fprintf(stderr, "you should use -c\n");
					exit(1);
				}
			}

			if(new_md != NULL && s->mac != NULL && \
				strncmp(new_md, s->mac, md_len*2) != 0)
			{
				printf("file: %s ", s->file);
				printf("old: %s ", s->mac);
				printf("new: %s ", new_md);
				printf("\n");

				if(*command != '\0') {
					system(command);
				}

			}

			if(s->next == NULL)
				break;
			s = s->next;
		}

		
		if(calculate_md)
			return 0;

		sleep(60);
	}



	return 0;


}
